/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.adl;

import java.util.HashMap;

import org.objectweb.fractal.adl.Factory;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.LifeCycleController;
import org.objectweb.fractal.util.Fractal;

/**
 * A class to launch a component from its definition from the command line.
 * Usage: <br/>
 * <code> Launcher [-factory=&lt;factory def&gt;] [-backend=&lt;backend def&gt;] &lt;definition&gt; [ &lt;itf&gt; ]</code>
 * <br/>Where &lt;definition&gt; is the name of the component to be instantiated
 * and started,and &lt;itf&gt; is the name of its Runnable interface, if it has
 * one.
 */
public final class Launcher
{

  private Launcher()
  {
  }

  /**
   * Entry point
   * 
   * @param args command line arguments
   * @throws Exception if error.
   */
  public static void main(final String[] args) throws Exception
  {
    String[] pargs = parseArgs(args);
    Object comp = createComponent(pargs);

    if (comp instanceof Component)
    {
      LifeCycleController lc = null;
      try
      {
        lc = Fractal.getLifeCycleController((Component) comp);
      }
      catch (NoSuchInterfaceException ignored)
      {
      }
      if (lc != null)
      {
        lc.startFc();
      }
      Runnable r = null;
      try
      {
        r = (Runnable) ((Component) comp).getFcInterface(pargs[2]);
      }
      catch (NoSuchInterfaceException ignored)
      {
      }
      if (r != null)
      {
        r.run();
      }
    }
    else
    {
      if (comp instanceof LifeCycleController)
      {
        ((LifeCycleController) comp).startFc();
      }
      if (comp instanceof Runnable)
      {
        ((Runnable) comp).run();
      }
    }
  }

  private static Object createComponent(String[] pargs) throws Exception
  {
    Factory f = org.objectweb.fractal.adl.FactoryFactory.getFactory(pargs[0],
        pargs[1], new HashMap());
    return f.newComponent(pargs[2], new HashMap());
  }

  private static String[] parseArgs(final String[] args)
  {
    if (args.length < 1 || args.length > 4)
    {
      parseError();
    }
    String[] result = new String[4];
    int i = 0;
    if (args[i].startsWith("-factory="))
    {
      result[0] = args[i].substring(9);
      i++;
    }
    else
    {
      result[0] = FactoryFactory.FACTORY_NAME;
    }

    if (args.length == i)
    {
      parseError();
    }
    if (args[i].startsWith("-backend="))
    {
      result[1] = args[i].substring(9);
      i++;
    }
    else
    {
      result[1] = FactoryFactory.FRACTAL_BACKEND;
    }

    if (args.length == i)
    {
      parseError();
    }
    result[2] = args[i];

    i++;
    if (args.length == i)
    {
      result[3] = "run";
    }
    else
    {
      result[3] = args[i];
    }
    if (args.length > i + 1)
    {
      parseError();
    }

    return result;
  }

  private static void parseError()
  {
    System.out.print("Usage: Launcher [-factory=<factory def>|-backend=");
    System.out.println("<backend def>] <definition> [ <itf> ]");
    System.out.print("where <definition> is the name of the component to be ");
    System.out.print("instantiated and started,\n      <itf> is the name of ");
    System.out.print("its Runnable interface, if it has one,\n      ");
    System.out.print("<factory def> is the name of the factory implementation");
    System.out.print(" to be used (if not specified use Dream implementation)");
    System.out.print(",\n      <backend def> is the name of the backend ");
    System.out.print("implementation to be used (if not specified use Fractal");
    System.out.print(" implementation for Dream factory)");
    System.exit(1);
  }

}