/**
 *
 */

package org.objectweb.util.monolog.wrapper.javaLog;

import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Handler;
import org.objectweb.util.monolog.wrapper.common.AbstractFactory;
import org.objectweb.util.monolog.wrapper.javaLog.Logger;
import org.objectweb.util.monolog.Monolog;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Properties;
import java.util.logging.ConsoleHandler;
import java.util.logging.FileHandler;
import java.util.logging.LogManager;

public class LoggerFactory
        extends AbstractFactory {

    protected static LogManager manager = null;
    protected static Logger rootLogger = null;

    /**
     * This static code initialize the value of the variable defined into
     * BasicLevel.
     */
    static {
        BasicLevel.INHERIT = -1;
        debug("INHERIT= " + BasicLevel.INHERIT);
        BasicLevel.DEBUG = java.util.logging.Level.FINEST.intValue();
        debug("DEBUG= " + BasicLevel.DEBUG);
        BasicLevel.INFO = java.util.logging.Level.INFO.intValue();
        debug("INFO= " + BasicLevel.INFO);
        BasicLevel.WARN = java.util.logging.Level.WARNING.intValue();
        debug("WARN= " + BasicLevel.WARN);
        BasicLevel.ERROR = java.util.logging.Level.SEVERE.intValue();
        debug("ERROR= " + BasicLevel.ERROR);
        BasicLevel.FATAL = java.util.logging.Level.SEVERE.intValue();
        debug("FATAL= " + BasicLevel.FATAL);

        BasicLevel.LEVEL_INHERIT = new LevelImpl("INHERIT", BasicLevel.INHERIT);
        BasicLevel.LEVEL_DEBUG = new LevelImpl("DEBUG", BasicLevel.DEBUG);
        BasicLevel.LEVEL_INFO = new LevelImpl("INFO", BasicLevel.INFO);
        BasicLevel.LEVEL_WARN = new LevelImpl("WARN", BasicLevel.WARN);
        BasicLevel.LEVEL_ERROR = new LevelImpl("ERROR", BasicLevel.ERROR);
        BasicLevel.LEVEL_FATAL = new LevelImpl("FATAL", BasicLevel.FATAL);

        manager = LogManager.getLogManager();
        rootLogger = new Logger(manager.getLogger(""));
    }

    /**
     * This constant is used to initialize the factory with the configure method
     */
    public final static String JAVALOG_CONFIGURATION = "javaLogConfiguration";

    /**
     * This constant means that this java log system must be initialize with
     * the default configuration
     */
    public final static String DEFAULT = "default";

    /**
     * This constant means that this java log system must be initialize with
     * a property file
     */
    public final static String PROPERTY = "property";

    /**
     * This constant means that this java log system must be initialize with
     * a xml file
     */
    public final static String CLASS = "class";

    /**
     * This constant is the properties file name with wich the java log system
     * must be initialized.
     */
    public final static String JAVALOG_CONFIGURATION_FILE
            = "javaLogConfigurationFile";

    /**
     * This constant is the properties class name with wich the java log system
     * must be initialized.
     */
    public final static String JAVALOG_CONFIGURATION_CLASS
            = "javaLogConfigurationClass";

    public LoggerFactory() {
        super();
    }

    /**
     * This method allocates org.objectweb.util.monolog.wrapper.javaLog.Logger
     * objects whic are also java.util.logging.Logger and
     * org.objectweb.util.monolog.api.Logger.
     */
    protected synchronized Logger getMonoLogger(String name, String resName) {
        if (name == null)
            throw new IllegalArgumentException(
                    "The specified Logger name is null");
        if (name.equals("root") || name.length()==0) {
            return rootLogger;
        }
        // Search if the logger already exist.
        Object o = manager.getLogger(name);
        if (o == null) {
            // It doesn't exist => creates and adds it
            Logger result = new Logger(name, resName);
            Monolog.debug("Instanciate the logger " + name);
            manager.addLogger(result);
            return result;
        } else if (!o.getClass().equals(Logger.class)) {
            throw new RuntimeException("Bad logger class (logger name: '"
                    + name +"'), expected: " + Logger.class
                    + ", found: " + o.getClass());
        } else {
            return (Logger) o;
        }
    }


    // IMPLEMENTATION OF THE Configurable INTERFACE //
    //----------------------------------------------//

    /**
     * This method permits to configure the factory.
     * The properties parameter must contains the JAVALOG_CONFIGURATION property.
     * Its value can be DEFAULT, PROPERTY or XML.
     * In the PROPERTY case of the properties parameter must also contain the
     * JAVALOG_CONFIGURATION_FILE property which the value is the configuration
     *  file name.
     * In the CLASS case of the properties parameter must also contain the
     * JAVALOG_CONFIGURATION_CLASS property which the value is the configuration
     *  class name which will initialized the java log system..
     *
     */
    public void configure(Properties prop) throws Exception {
        if (prop == null)
            return;
        String confMode = prop.getProperty(JAVALOG_CONFIGURATION, null);
        if (confMode == null)
            return;

        String param = null;
        if (confMode.equals(PROPERTY)) {
            param = prop.getProperty(JAVALOG_CONFIGURATION_FILE, null);
            if (param != null)
                System.setProperty("java.util.logging.config.file", param);
            manager.readConfiguration();
        } else if (confMode.equals(CLASS)) {
            param = prop.getProperty(JAVALOG_CONFIGURATION_CLASS, null);
            if (param != null)
                System.setProperty("java.util.logging.config.class", param);
            manager.readConfiguration();
        }
    }


    // IMPLEMENTATION OF LoggerFactory INTERFACE //
    //-------------------------------------------//

    public org.objectweb.util.monolog.api.Logger getLogger(String key) {
        return getMonoLogger(key, resourceBundleName);
    }

    public org.objectweb.util.monolog.api.Logger getLogger(String key, String rbn) {
        return getMonoLogger(key, rbn);
    }

    /**
     * It retrieves a list of all loggers.
     */
    public org.objectweb.util.monolog.api.Logger[] getLoggers() {
        ArrayList res = new ArrayList();
        for (Enumeration e = manager.getLoggerNames(); e.hasMoreElements();) {
            res.add(manager.getLogger((String) e.nextElement()));
        }
        return (Logger[]) res.toArray(new Logger[0]);
    }

    // IMPLEMENTATION OF THE HandlerFactory INTERFACE //
    //------------------------------------------------//

    public Handler createHandler(String hn, byte ht) {
        Handler res = (Handler) handlers.get(hn);
        if (res != null) {
            return res;
        }
        switch (ht) {
        case Handler.CONSOLE_HANDLER_TYPE:
            ConsoleHandler ch = new ConsoleHandler();
            ch.setLevel(java.util.logging.Level.FINEST);
            res = new GenericHandler(hn, ch, ht);
            break;
        case Handler.FILE_HANDLER_TYPE:
            try {
                FileHandler fh = new FileHandler();
                fh.setLevel(java.util.logging.Level.FINEST);
                res = new GenericHandler(hn, fh, ht);
            } catch (Exception e) {
                AbstractFactory.debug(e.getMessage());
                return null;
            }
            break;
        case Handler.ROLLING_FILE_HANDLER_TYPE:
            return null;
            //res = new GenericHandler(hn, RollingFileHandler(hn), ht);
            //break;
        case Handler.GENERIC_HANDLER_TYPE:
            res = new GenericHandler(hn, ht);
            break;
        default:
            return null;
        }
        handlers.put(hn, res);
        return res;
    }
}
